import pytest
import shlex


def test_sles_ec2_dracut_conf(host, get_release_value, determine_architecture):
    if determine_architecture() != 'X86_64':
        pytest.skip('Only x86_64 architecture is tested.')

    needed_drivers = [
        'ena',
        'virtio',
        'virtio_scsi',
        'xen-blkfront',
        'xen-netfront',
        'nvme',
        'nvme-core'
    ]

    base_dracut_config_dirs = [
        '/usr/lib/dracut/dracut.conf.d/',
        '/etc/dracut.conf.d/'
    ]
    config_files = []
    for config_dir in base_dracut_config_dirs:
        cfg_dir = host.file(config_dir)
        if not cfg_dir.exists or not cfg_dir.is_directory:
            continue

        config_dir_ls = host.run(f'ls {config_dir}').stdout.strip()
        config_files_in_dir = shlex.split(config_dir_ls)
        for config_file in config_files_in_dir:
            config_files.append(config_dir + config_file)

    version = get_release_value('VERSION')
    assert version

    name = get_release_value('PRETTY_NAME').lower()
    assert name

    for driver in needed_drivers:
        if (
            driver.startswith('virtio') and
            (
                version.startswith('15') or
                version == '12-SP5' or
                'micro' in name
            )
        ):
            continue

        found = False
        for cfg_filename in config_files:
            dracut_conf = host.file(cfg_filename)
            if (
                dracut_conf.exists and
                dracut_conf.is_file and
                dracut_conf.contains(driver)
            ):
                found = True
                break
        assert found



import shlex


def test_sles_grow_root(host):
    """
    Ensure root filesystem grows to non default size

    The partitions are slightly different in all images.
    However, there is always a root partition and in Azure
    the boot partition is 1GB.

    All the smaller partitions round to zero and
    therefore they are ignored in calculating size of fs.
    """

    # Get root disk size
    root_part = host.run(
        'findmnt -v -n -f -o SOURCE /'
    ).stdout.strip()

    device_name = host.run(
        'lsblk -npo pkname {part}'.format(part=root_part)
    ).stdout.strip()

    result = host.run(
        'lsblk -ndo size {device}'.format(device=device_name)
    ).stdout.strip()

    try:
        disk_size = float(result.replace('G', ''))
        unit = 'G'
    except ValueError:
        disk_size = float(result.replace('T', ''))
        unit = 'T'

    # Get root partition size
    result = host.run(
        'df -B{unit} {part} | sed 1D'.format(unit=unit, part=root_part)
    ).stdout.strip()

    root_size = shlex.split(result)[1]  # Filesystem 1G-blocks
    root_size = float(root_size.replace(unit, ''))

    # Get boot partition size
    boot_part = host.run('findmnt -v -n -f -o SOURCE /boot').stdout.strip()

    if boot_part:
        # Some images have a separate boot partition that is >= 1G
        result = host.run(
            'df -B{unit} {part} | sed 1D'.format(unit=unit, part=boot_part)
        ).stdout.strip()

        boot_size = shlex.split(result)[1]  # Filesystem 1G-blocks
        boot_size = float(boot_size.replace(unit, ''))
    else:
        boot_size = 0

    # Get /var partition size
    var_part = host.run('findmnt -v -n -f -o SOURCE /var').stdout.strip()

    if var_part and var_part != root_part:
        # Some images have a separate /var partition that is >= 1G
        result = host.run(
            'df -B{unit} {part} | sed 1D'.format(unit=unit, part=var_part)
        ).stdout.strip()

        var_size = shlex.split(result)[1]  # Filesystem 1G-blocks
        var_size = float(var_size.replace(unit, ''))
    else:
        var_size = 0

    total_size = root_size + boot_size + var_size

    # Rounding can lead to small discrepancies
    assert (((disk_size - 1) <= total_size) and (total_size <= (disk_size + 1)))



# -*- coding: utf-8 -*-

"""Module for testing instances using SSH only."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from img_proof.ipa_exceptions import SSHCloudException
from img_proof.ipa_cloud import IpaCloud


class SSHCloud(IpaCloud):
    """Class for testing instances in Azure."""
    cloud = 'ssh'

    def post_init(self):
        """Initialize SSH cloud class."""

        # Cannot cleanup SSH instance
        self.cleanup = False
        self.instance_ip = self.custom_args.get('ip_address')

        if not self.instance_ip:
            raise SSHCloudException(
                'IP address is required to connect to instance.'
            )

        if not self.ssh_private_key_file:
            raise SSHCloudException(
                'SSH private key file is required to connect to instance.'
            )

        if not self.ssh_user:
            raise SSHCloudException(
                'SSH user is required to connect to instance.'
            )

    def _is_instance_running(self):
        """
        Return True.

        Instance must be running with SSh cloud.
        """
        return True

    def _get_instance_state(self):
        """
        Do nothing.
        """
        pass

    def _get_instance(self):
        """
        Do nothing.
        """
        pass

    def get_console_log(self):
        """
        Return console log output if it is available.
        """
        return ''  # No console log for SSH backend

    def _launch_instance(self):
        """
        Do nothing.

        SSH instance must be running.
        """
        raise SSHCloudException(
            'SSH class cannot launch instances.'
        )

    def _set_image_id(self):
        """
        Do nothing.
        """
        raise SSHCloudException(
            'SSH class has no access to cloud API.'
        )

    def _set_instance_ip(self):
        """
        Do nothing.
        """
        raise SSHCloudException(
            'SSH class has no access to cloud API.'
        )

    def _start_instance(self):
        """
        Do nothing.
        """
        raise SSHCloudException(
            'SSH class has no access to cloud API.'
        )

    def _stop_instance(self):
        """
        Do nothing.
        """
        raise SSHCloudException(
            'SSH class has no access to cloud API.'
        )

    def _terminate_instance(self):
        """
        Do nothing.
        """
        raise SSHCloudException(
            'SSH class has no access to cloud API.'
        )



import pytest
import re


@pytest.mark.skipinbeta
def test_sles_kernel_version(host, get_release_value, is_sle_micro):
    version = get_release_value('VERSION')
    assert version

    if version in ('11.4', '12-SP1', '12-SP2', '12-SP3'):
        pytest.skip('Whoops! Image does not have version in kernel config.')

    if is_sle_micro():
        pytest.skip('Micro has product version instead of SLE version.')

    match = re.match(r'^(\d+)(?:[.-](?:SP)?(\d+))?$', version)
    assert match, f"Unexpected version format: {version}"

    major = match.group(1)
    patchlevel = match.group(2)

    config = host.run('sudo zcat /proc/config.gz')
    assert config.rc == 0, "Failed to read kernel config"

    assert f'CONFIG_SUSE_VERSION={major}\n' in config.stdout

    if patchlevel is not None:
        assert f'CONFIG_SUSE_PATCHLEVEL={patchlevel}\n' in config.stdout



import ipaddress
import pytest
import shlex


def test_sles_smt_reg(
    check_cloud_register,
    determine_provider,
    get_smt_server_name,
    get_smt_servers,
    host
):
    provider = determine_provider()

    # ensure instance registered successfully
    assert check_cloud_register()

    # ensure the correct smt/rmt ip is used
    servers = get_smt_servers(provider)
    smt_ips = list(filter(None, [server['ip'] for server in servers]))
    smt_ips += list(filter(None, [server['ipv6'] for server in servers]))

    result = host.run(
        'cat /etc/hosts | grep %s' % get_smt_server_name(provider)
    )
    smt_ip = shlex.split(result.stdout)[0]

    for server_ip in smt_ips:
        if ipaddress.ip_address(smt_ip) == ipaddress.ip_address(server_ip):
            break
    else:
        pytest.fail(f'RMT Server not found {smt_ip}')

    # ensure region hint is in log
    cloud_register_log = host.file('/var/log/cloudregister')
    assert cloud_register_log.contains(
        'INFO:Region server arguments: ?regionHint='
    )



# -*- coding: utf-8 -*-

"""Red Hat distro family module and sync points."""

# Copyright (c) 2020 Neal Gompa. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from img_proof.ipa_distro import Distro
from img_proof.ipa_exceptions import IpaRedHatException


class RedHat(Distro):
    """Red Hat distro class."""

    def get_install_cmd(self):
        """Return install package command for Red Hat distributions."""
        return 'dnf --assumeyes --nogpgcheck install'

    def get_refresh_repo_cmd(self):
        """Return refresh repo command for Red Hat distributions."""
        return 'dnf --assumeyes makecache'

    def get_stop_ssh_service_cmd(self):
        """
        Return command to stop SSH service for Red Hat distributions.

        SSH stop command determined by init system.
        """
        if self.init_system == 'systemd':
            return 'systemctl stop sshd.service'
        elif self.init_system == 'init':
            return 'service sshd stop'
        else:
            raise IpaRedHatException(
                'The init system for this Red Hat system cannot be determined.'
            )

    def get_update_cmd(self):
        """Return command to update Red Hat distribution instance."""
        return 'dnf --assumeyes upgrade'



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""img_proof utils unit tests."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import time

import paramiko
import pytest

from shutil import copyfile
from tempfile import NamedTemporaryFile

from img_proof import ipa_utils
from img_proof.ipa_exceptions import IpaSSHException, IpaUtilsException

from unittest.mock import MagicMock, patch

LOCALHOST = '127.0.0.1'


@patch('img_proof.ipa_utils.execute_ssh_command')
def test_utils_clear_cache(mock_exec_cmd):
    """Test img_proof utils client cache and clear specific ip."""
    client = MagicMock()
    ipa_utils.CLIENT_CACHE[LOCALHOST] = client

    val = ipa_utils.get_ssh_client(LOCALHOST, 'tests/data/ida_test')
    assert client == val

    # Test clear specfic IP
    ipa_utils.clear_cache(LOCALHOST)
    with pytest.raises(KeyError):
        ipa_utils.CLIENT_CACHE[LOCALHOST]

    ipa_utils.CLIENT_CACHE[LOCALHOST] = client

    # Test clear all cache
    ipa_utils.clear_cache()
    with pytest.raises(KeyError):
        ipa_utils.CLIENT_CACHE[LOCALHOST]


@patch.object(paramiko.SSHClient, 'exec_command')
@patch.object(paramiko.SSHClient, 'connect')
def test_utils_get_ssh_connection(mock_connect, mock_exec_cmd):
    """Test successful ssh connection."""
    stdin = stdout = stderr = MagicMock()
    stderr.read.return_value = b''

    mock_connect.return_value = None
    mock_exec_cmd.return_value = (stdin, stdout, stderr)

    ipa_utils.get_ssh_client(LOCALHOST, 'tests/data/ida_test')
    assert mock_connect.call_count == 1
    assert mock_exec_cmd.call_count == 1

    # Clear cache for subsequent tests.
    ipa_utils.clear_cache()


@patch.object(time, 'time')
@patch.object(time, 'sleep')
@patch.object(paramiko.SSHClient, 'connect')
def test_utils_ssh_connect_exception(mock_connect, mock_sleep, mock_time):
    """Test exception raised connecting to ssh."""
    mock_connect.side_effect = paramiko.ssh_exception.SSHException('ERROR!')
    mock_sleep.return_value = None
    mock_time.side_effect = [0, 0, 2]

    with pytest.raises(IpaSSHException) as error:
        ipa_utils.get_ssh_client(
            LOCALHOST,
            'tests/data/ida_test',
            timeout=1
        )

    assert str(error.value) == 'Attempt to establish SSH connection failed.'
    assert mock_connect.call_count > 0


@patch.object(time, 'time')
@patch.object(time, 'sleep')
@patch.object(paramiko.SSHClient, 'connect')
@patch.object(paramiko.SSHClient, 'exec_command')
def test_utils_ssh_connect_authexception_once(
    mock_exec_cmd,
    mock_connect,
    mock_sleep,
    mock_time
):
    """Test auth exception raised happening once connecting to ssh."""
    mock_connect.side_effect = [
        paramiko.ssh_exception.AuthenticationException,
        True
    ]
    stdin = stdout = stderr = MagicMock()
    stderr.read.return_value = b''
    mock_exec_cmd.return_value = (stdin, stdout, stderr)
    mock_sleep.return_value = None
    mock_time.side_effect = [0, 0, 2]

    ipa_utils.get_ssh_client(
        LOCALHOST,
        'tests/data/ida_test',
        timeout=1
    )
    assert mock_connect.call_count == 2
    assert mock_exec_cmd.call_count == 1

    ipa_utils.get_ssh_client(
        LOCALHOST,
        'tests/data/ida_test',
        timeout=10
    )


@patch.object(time, 'time')
@patch.object(time, 'sleep')
@patch.object(paramiko.SSHClient, 'connect')
def test_utils_ssh_connect_authexception_twice(
    mock_connect,
    mock_sleep,
    mock_time
):
    """Test auth exception raised happening once connecting to ssh."""
    mock_connect.side_effect = [
        paramiko.ssh_exception.AuthenticationException,
        paramiko.ssh_exception.AuthenticationException
    ]
    mock_sleep.return_value = None
    mock_time.side_effect = [0, 0, 2]

    with pytest.raises(IpaSSHException) as error:
        ipa_utils.get_ssh_client(
            LOCALHOST,
            'tests/data/ida_test',
            timeout=1
        )

    assert str(error.value) == \
        'Authentication failed while establishing SSH connection.'


def test_utils_ssh_exec_command():
    """Test successful command execution."""
    stdin = MagicMock()
    stdin.read.return_value = ''

    stderr = MagicMock()
    stderr.read.return_value = ''

    stdout = MagicMock()
    stdout.read.return_value = b'test test.sh'

    client = MagicMock()
    client.exec_command.return_value = (stdin, stdout, stderr)

    out = ipa_utils.execute_ssh_command(client, 'ls')
    assert out == 'test test.sh'


def test_utils_ssh_exec_command_exception():
    """Test error in ssh exec command raises exception."""
    stdin = MagicMock()

    stderr = MagicMock()
    stderr.read.return_value = b'Exception: ls is not allowed!'

    stdout = MagicMock()
    stdout.read.return_value = b'Other information\n'

    client = MagicMock()
    client.exec_command.return_value = (stdin, stdout, stderr)

    with pytest.raises(IpaSSHException) as error:
        ipa_utils.execute_ssh_command(client, 'ls')

    msg = 'Other information\nException: ls is not allowed!'
    assert str(error.value) == msg


def test_utils_expand_test_files():
    """Test expand test files utils function."""
    test_dirs = ['tests/data/tests']
    names = ['test_sles::test_sles', 'test_image_desc']
    exclude = []
    expanded = ipa_utils.expand_test_files(test_dirs, names, exclude)

    assert 'tests/data/tests/test_image.py' in expanded
    assert 'tests/data/tests/test_sles.py::test_sles' in expanded


def test_utils_expand_test_files_exclude():
    """Test expand test files utils function."""
    test_dirs = ['tests/data/tests']
    names = ['test_sles::test_sles', 'test_image_desc']
    exclude = ['test_image']
    expanded = ipa_utils.expand_test_files(test_dirs, names, exclude)

    assert 'tests/data/tests/test_image.py' not in expanded


def test_utils_test_file_not_found():
    """Test expand test file does not exist raises exception."""
    test_dirs = ['tests/data/tests']
    names = ['test_fake']
    exclude = []

    with pytest.raises(IpaUtilsException) as error:
        ipa_utils.expand_test_files(test_dirs, names, exclude)

    assert str(error.value) == \
        'Test file with name: test_fake cannot be found.'


def test_utils_expand_names_not_list():
    """Test exception raised if names not a list."""
    test_dirs = ['tests/data/tests']
    names = 'test_sles'
    exclude = []

    with pytest.raises(IpaUtilsException) as error:
        ipa_utils.expand_test_files(test_dirs, names, exclude)
    assert str(error.value) == \
        'Names must be a list containing test names and/or test descriptions.'


def test_utils_expand_exclude_string():
    """Test exclude handled as a str."""
    test_dirs = ['tests/data/tests']
    names = ['test_sles']
    exclude = 'test_image'

    expanded = ipa_utils.expand_test_files(test_dirs, names, exclude)
    assert 'tests/data/tests/test_image.py' not in expanded


def test_utils_expand_sync_points():
    """Test expand test files with sync points."""
    test_dirs = ['tests/data/tests']
    names = ['test_soft_reboot', 'test_sles_desc', 'test_hard_reboot']
    exclude = []
    expanded = ipa_utils.expand_test_files(test_dirs, names, exclude)

    assert len(expanded) == 4
    assert expanded[0] == 'test_soft_reboot'
    assert expanded[1] == 'tests/data/tests/test_sles.py'
    assert expanded[2] == 'tests/data/tests/test_image.py'
    assert expanded[3] == 'test_hard_reboot'


@patch('img_proof.ipa_utils.execute_ssh_command')
def test_utils_extract_archive(mock_exec_ssh_command):
    client = MagicMock()

    mock_exec_ssh_command.return_value = 'archive successfully extracted!'

    # Test gzip
    ipa_utils.extract_archive(client, 'archive.tar.gz', extract_path='/tmp/')
    mock_exec_ssh_command.assert_called_once_with(
        client, 'tar -xf archive.tar.gz -C /tmp/'
    )
    mock_exec_ssh_command.reset_mock()


def test_utils_duplicate_files():
    """Test exception raised if duplicate test files exist."""
    test_dirs = ['tests/data/tests', 'tests/data/tests2']

    with pytest.raises(IpaUtilsException) as error:
        tests, descriptions = ipa_utils.get_test_files(test_dirs)
    assert (
        'Duplicate test file name found: '
        'tests/data/tests2/test_image.py, tests/data/tests/test_image.py' in
        str(error.value)
    )


def test_utils_file_description_overlap():
    """Test exception raised if test file and test description share name."""
    test_dirs = ['tests/data/tests3']

    with pytest.raises(IpaUtilsException) as error:
        tests, descriptions = ipa_utils.get_test_files(test_dirs)
    assert (
        'Test description name matches test file: '
        'tests/data/tests3/test_image.yaml, tests/data/tests3/test_image.py' in
        str(error.value)
    )


def test_utils_duplicate_test_description():
    """Test exception raised if duplicate test description."""
    test_dirs = ['tests/data/tests', 'tests/data/tests4']

    with pytest.raises(IpaUtilsException) as error:
        tests, descriptions = ipa_utils.get_test_files(test_dirs)
    assert (
        'Duplicate test description file name found: '
        'tests/data/tests4/test_image_desc.yaml, '
        'tests/data/tests/test_image_desc.yaml' in
        str(error.value)
    )


def test_get_config_values():
    """Test utils get config values function."""
    data = ipa_utils.get_config_values(
        'tests/data/config',
        'ec2',
        'img_proof'
    )

    assert data['region'] == 'us-west-1'
    assert data['test_dirs'] == 'tests/data/tests'


def test_utils_get_config_values_exceptions():
    """Test utils get config values function invalid path."""
    with pytest.raises(IpaUtilsException) as error:
        ipa_utils.get_config_values(
            'tests/data/fake.config',
            'ec2',
            'img_proof'
        )

    assert str(error.value) == \
        'Config file not found: tests/data/fake.config'


def test_utils_get_host_key_fingerprint():
    """Test get host key fingerprint function."""
    client = MagicMock()
    transport = MagicMock()
    key = MagicMock()

    client.get_transport.return_value = transport
    transport.get_remote_server_key.return_value = key
    key.get_fingerprint.return_value = b'\x04\x82\x04\x82'

    assert ipa_utils.get_host_key_fingerprint(client) == b'04820482'


def test_utils_get_random_string():
    """Test get random string function returns default len 12 string."""
    rand_string = ipa_utils.get_random_string()
    assert len(rand_string) == 12
    assert isinstance(rand_string, str)


def test_utils_put_file():
    client = MagicMock()
    sftp_client = MagicMock()
    client.open_sftp.return_value = sftp_client

    source_file = '/home/user/temp.file'
    destination_file = 'temp.file'

    ipa_utils.put_file(client, source_file, destination_file)

    sftp_client.put.assert_called_once_with(source_file, destination_file)
    sftp_client.close.assert_called_once_with()


def test_utils_redirect_output():
    """Test redirect output context manager."""
    temp_file = NamedTemporaryFile(mode='w+', delete=False)

    with ipa_utils.redirect_output(temp_file):
        print('Test message to file!')

    with open(temp_file.name) as temp_file:
        assert temp_file.readline() == 'Test message to file!\n'

    with ipa_utils.ignored(OSError):
        os.remove(temp_file.name)


def test_utils_ssh_config_context_mgr():
    """Test ssh config context manager function."""
    with ipa_utils.ssh_config('root', 'tests/data/ida_test') as conf:
        assert os.path.isfile(conf)
        with open(conf, 'r') as conf_file:
            assert conf_file.readline() == 'Host *\n'
            assert conf_file.readline() == \
                '    IdentityFile tests/data/ida_test\n'
            assert conf_file.readline() == '    User root'

    assert not os.path.isfile(conf)


def test_utils_history_log():
    """Test utils history log function."""
    history_file = NamedTemporaryFile(delete=False)
    history_file.close()

    copyfile('tests/data/.history', history_file.name)

    ipa_utils.update_history_log(
        history_file.name,
        test_log='tests/data/history.log'
    )

    with open(history_file.name, 'r+') as f:
        lines = f.readlines()

    assert lines[-1].strip() == 'tests/data/history.log'

    with ipa_utils.ignored(OSError):
        os.remove(history_file.name)


def test_utils_history_log_no_input():
    """Test utils history log function raises if no input."""
    with pytest.raises(IpaUtilsException) as error:
        ipa_utils.update_history_log('tests/data/.history')

    assert str(error.value) == \
        'A test log or clear flag must be provided.'


def test_utils_generate_instance_name():
    """Test generate instance name method."""
    name = ipa_utils.generate_instance_name('azure-img-proof-test')
    assert len(name) == 26
    assert name.startswith('azure-img-proof-test-')



import pytest


@pytest.mark.skipinbeta
def test_sles_sap_motd(host, get_release_value, get_variant):
    motd = host.file('/etc/motd')
    assert motd.exists
    assert motd.is_file

    variant = get_variant()
    if 'hardened' in variant:
        pytest.skip('Unable to validate motd in hardened images.')

    assert motd.contains('for SAP Applications')



# -*- coding: utf-8 -*-

"""Collect test items plugin."""

# Copyright (c) 2019 SUSE LLC
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

from collections import defaultdict


class CollectItemsPlugin(object):
    """Collect test items pytest plugin class."""

    def __init__(self):
        """Intialize collected dictionary."""
        self.collected = defaultdict(list)

    def pytest_collection_modifyitems(self, items):
        """
        Plugin to return a list of test cases from pytest collection.

        The list returned contains all test cases and classes in each
        test file found.
        """
        for item in items:
            test_class = None
            try:
                path, test_class, parens, test_case = item.nodeid.split('::')
            except ValueError:
                path, test_case = item.nodeid.split('::')

            test_file = path.split(os.sep)[-1].replace('.py', '')

            # Filter None and empty string values
            self.collected[test_file].append(
                '::'.join(filter(None, [test_file, test_class, test_case]))
            )



# -*- coding: utf-8 -*-

"""Controller class for img_proof endpoints."""

# Copyright (c) 2020 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import os
import pytest
import shlex

from img_proof.collect_items import CollectItemsPlugin
from img_proof.ipa_azure import AzureCloud
from img_proof.ipa_constants import TEST_PATHS
from img_proof.ipa_ec2 import EC2Cloud
from img_proof.ipa_exceptions import IpaControllerException
from img_proof.ipa_gce import GCECloud
from img_proof.ipa_ssh import SSHCloud
from img_proof.ipa_aliyun import AliyunCloud
from img_proof.ipa_utils import get_test_files


def test_image(
    cloud_name,
    accelerated_networking=None,
    access_key_id=None,
    access_key=None,
    access_secret=None,
    account=None,
    cleanup=None,
    config=None,
    description=None,
    distro=None,
    early_exit=None,
    history_log=None,
    image_id=None,
    image_project=None,
    inject=None,
    instance_type=None,
    ip_address=None,
    log_level=None,
    no_default_test_dirs=None,
    cloud_config=None,
    region=None,
    results_dir=None,
    running_instance_id=None,
    secret_access_key=None,
    security_group_id=None,
    service_account_file=None,
    ssh_key_name=None,
    ssh_private_key_file=None,
    ssh_user=None,
    subnet_id=None,
    test_dirs=None,
    tests=None,
    timeout=None,
    vnet_name=None,
    vnet_resource_group=None,
    collect_vm_info=None,
    enable_secure_boot=None,
    enable_uefi=None,
    log_callback=None,
    prefix_name=None,
    retry_count=None,
    v_switch_id=None,
    sev_capable=None,
    additional_info=None,
    use_gvnic=None,
    root_disk_size=None,
    gallery_name=None,
    gallery_resource_group=None,
    image_version=None,
    architecture=None,
    beta=None,
    exclude=None,
    cpu_options=None,
    instance_options=None,
    include_plan_information=None,
):
    """Creates a cloud framework instance and initiates testing."""
    kwargs = {
        'cleanup': cleanup,
        'config': config,
        'description': description,
        'distro_name': distro,
        'early_exit': early_exit,
        'history_log': history_log,
        'image_id': image_id,
        'inject': inject,
        'instance_type': instance_type,
        'log_level': log_level,
        'no_default_test_dirs': no_default_test_dirs,
        'cloud_config': cloud_config,
        'region': region,
        'results_dir': results_dir,
        'running_instance_id': running_instance_id,
        'ssh_private_key_file': ssh_private_key_file,
        'ssh_user': ssh_user,
        'subnet_id': subnet_id,
        'test_dirs': test_dirs,
        'test_files': tests,
        'timeout': timeout,
        'collect_vm_info': collect_vm_info,
        'enable_secure_boot': enable_secure_boot,
        'enable_uefi': enable_uefi,
        'log_callback': log_callback,
        'prefix_name': prefix_name,
        'retry_count': retry_count,
        'root_disk_size': root_disk_size,
        'beta': beta,
        'exclude': exclude,
        'instance_options': instance_options
    }

    cloud_name = cloud_name.lower()
    if cloud_name == 'azure':
        kwargs['custom_args'] = {
            'accelerated_networking': accelerated_networking,
            'service_account_file': service_account_file,
            'vnet_name': vnet_name,
            'vnet_resource_group': vnet_resource_group,
            'gallery_name': gallery_name,
            'gallery_resource_group': gallery_resource_group,
            'image_version': image_version,
            'include_plan_info': include_plan_information,
        }
        cloud = AzureCloud(**kwargs)
    elif cloud_name == 'ec2':
        kwargs['custom_args'] = {
            'access_key_id': access_key_id,
            'account_name': account,
            'secret_access_key': secret_access_key,
            'security_group_id': security_group_id,
            'ssh_key_name': ssh_key_name,
            'additional_info': additional_info,
            'cpu_options': cpu_options
        }
        cloud = EC2Cloud(**kwargs)
    elif cloud_name == 'gce':
        kwargs['custom_args'] = {
            'service_account_file': service_account_file,
            'image_project': image_project,
            'sev_capable': sev_capable,
            'use_gvnic': use_gvnic,
            'architecture': architecture
        }
        cloud = GCECloud(**kwargs)
    elif cloud_name == 'ssh':
        kwargs['custom_args'] = {'ip_address': ip_address}
        cloud = SSHCloud(**kwargs)
    elif cloud_name == 'aliyun':
        kwargs['custom_args'] = {
            'access_key': access_key,
            'access_secret': access_secret,
            'security_group_id': security_group_id,
            'v_switch_id': v_switch_id,
            'ssh_key_name': ssh_key_name
        }
        cloud = AliyunCloud(**kwargs)
    else:
        raise IpaControllerException(
            'Cloud framework: %s unavailable.' % cloud_name
        )

    return cloud.test_image()


def collect_results(results_file):
    """Return the result (pass/fail) for json file."""
    with open(results_file, 'r') as results:
        data = json.load(results)
    return data


def collect_tests(test_dirs=TEST_PATHS, verbose=False):
    """Return a list of test files and/or tests cases."""
    if test_dirs:
        test_dirs = [
            test_dir for test_dir in test_dirs if os.path.exists(test_dir)
        ]

    if not test_dirs:
        raise IpaControllerException(
            'No test directories found.'
        )

    if verbose:
        plugin = CollectItemsPlugin()
        args = '--collect-only -p no:terminal {}'.format(
            ' '.join(test_dirs)
        )
        cmds = shlex.split(args)
        pytest.main(cmds, plugins=[plugin])

        return plugin.collected.values()
    else:
        tests, descriptions = get_test_files(test_dirs)
        all_tests = list(tests.keys()) + list(descriptions.keys())
        return all_tests



# -*- coding: utf-8 -*-

"""Utility methods for img-proof cli endpoints."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img-proof. img-proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import shlex
import shutil
import sys

import click

from img_proof.ipa_controller import collect_results
from img_proof.ipa_utils import parse_test_name, update_history_log


def archive_history_item(item, destination, no_color):
    """
    Archive the log and results file for the given history item.

    Copy the files and update the results file in destination directory.
    """
    log_src, description = split_history_item(item.strip())

    # Get relative path for log:
    # {provider}/{image}/{instance}/{timestamp}.log
    log_dest = os.path.sep.join(log_src.rsplit(os.path.sep, 4)[1:])

    # Get results src and destination based on log paths.
    results_src = log_src.rsplit('.', 1)[0] + '.results'
    results_dest = log_dest.rsplit('.', 1)[0] + '.results'

    destination_path = os.path.join(destination, log_dest)
    log_dir = os.path.dirname(destination_path)

    try:
        if not os.path.isdir(log_dir):
            os.makedirs(log_dir)

        # Copy results and log files to archive directory.
        shutil.copyfile(log_src, destination_path)
        shutil.copyfile(results_src, os.path.join(destination, results_dest))
    except Exception as error:
        echo_style(
            'Unable to archive history item: %s' % error,
            no_color,
            fg='red'
        )
        sys.exit(1)
    else:
        # Only update the archive results log if no error occur.
        update_history_log(
            os.path.join(destination, '.history'),
            description=description,
            test_log=log_dest
        )


def cli_process_cpu_options(cxt, param, value):
    cpu_options = {}
    if value:
        try:
            cpu_options.update(
                dict(option.split('=', 1) for option in value.split(","))
            )
        except Exception as e:
            raise click.BadParameter(
                "--cpu-options parameter is invalid: %s" % e +
                '. (example value: "--cpu-options AmdSevSnp=enabled")'
            )
    return cpu_options


def echo_log(log_file, no_color):
    try:
        with open(log_file, 'r') as f:
            log_output = ''.join(f.readlines())
    except Exception as error:
        echo_style(
            'Unable to open results log file: %s' % error,
            no_color,
            fg='red'
        )
        sys.exit(1)

    click.echo(log_output)


def echo_results(data, no_color, verbose=False):
    """Print test results in nagios style format."""
    try:
        summary = data['summary']
    except KeyError as error:
        click.secho(
            'The results json is missing key: %s' % error,
            fg='red'
        )
        sys.exit(1)

    if 'failed' in summary or 'error' in summary:
        fg = 'red'
        status = 'FAILED'
    else:
        fg = 'green'
        status = 'PASSED'

    results = '{} tests={}|pass={}|skip={}|fail={}|error={}'.format(
        status,
        str(summary.get('total', 0)),
        str(summary.get('passed', 0)),
        str(summary.get('skipped', 0)),
        str(summary.get('failed', 0)),
        str(summary.get('error', 0))
    )
    echo_style(results, no_color, fg=fg)

    if verbose:
        echo_verbose_results(data, no_color)


def echo_results_file(results_file, no_color, verbose=False):
    """Print test results in nagios style format."""
    try:
        data = collect_results(results_file)
    except ValueError:
        echo_style(
            'The results file is not the proper json format.',
            no_color,
            fg='red'
        )
        sys.exit(1)
    except Exception as error:
        echo_style(
            'Unable to process results file: %s' % error,
            no_color,
            fg='red'
        )
        sys.exit(1)

    echo_results(data, no_color, verbose)


def echo_style(message, no_color, fg='yellow'):
    if no_color:
        click.echo(message)
    else:
        click.secho(message, fg=fg)


def echo_verbose_results(data, no_color):
    """Print list of tests and result of each test."""
    click.echo()
    click.echo(
        '\n'.join(
            '{}: {}'.format(key, val) for key, val in data['info'].items()
        )
    )
    click.echo()
    for test in data['tests']:
        if test['outcome'] == 'passed':
            fg = 'green'
        elif test['outcome'] == 'skipped':
            fg = 'yellow'
        else:
            fg = 'red'

        name = parse_test_name(test['nodeid'])
        echo_style(
            '{} {}'.format(name, test['outcome'].upper()),
            no_color,
            fg=fg
        )


def get_log_file_from_item(history):
    """
    Return the log file based on provided history item.

    Description is optional.
    """
    try:
        log_file, description = shlex.split(history)
    except ValueError:
        log_file = history.strip()

    return log_file


def results_history(history_log, no_color):
    """Display a list of img_proof test results history."""
    try:
        with open(history_log, 'r') as f:
            lines = f.readlines()
    except Exception as error:
        echo_style(
            'Unable to process results history log: %s' % error,
            no_color,
            fg='red'
        )
        sys.exit(1)

    index = len(lines)
    for item in lines:
        click.echo('{} {}'.format(index, item), nl=False)
        index -= 1


def split_history_item(history):
    """
    Return the log file and optional description for item.
    """
    try:
        log_file, description = shlex.split(history)
    except ValueError:
        log_file = history.strip()
        description = None

    return log_file, description



import shlex


def test_sles_switch_smt(get_smt_server_name,
                         determine_provider,
                         determine_region,
                         get_smt_servers,
                         host):
    """
    This is a helper function for SMT failover test.

    It is cast as a test to be easily included in test suite.
    """
    provider = determine_provider()
    region = determine_region(provider)

    result = host.run(
        'cat /etc/hosts | grep %s' % get_smt_server_name(provider)
    )
    smt_ip = shlex.split(result.stdout)[0]

    servers = get_smt_servers(provider, region)

    for server in servers:
        if server['ip'] != smt_ip:
            result = host.run(
                "sudo sed -i 's/{current}/{new}/' /etc/hosts".format(
                    current=smt_ip,
                    new=server['ip']
                )
            )
            assert result.rc == 0
            break



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""Pytest results plugin tests."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import shlex

import pytest

from img_proof.collect_items import CollectItemsPlugin


def test_collect_items():
    """Test collect test items."""
    plugin = CollectItemsPlugin()
    args = '--collect-only tests/data/tests'
    cmds = shlex.split(args)

    pytest.main(cmds, plugins=[plugin])
    assert len(plugin.collected.values()) == 4



import pytest


@pytest.mark.skipinbeta
def test_sles_motd(
    host,
    get_release_value,
    is_byos,
    is_suma_proxy,
    get_variant
):
    motd = host.file('/etc/motd')

    if not motd.exists:
        motd = host.file('/usr/lib/motd.d/10_header')

    assert motd.exists
    assert motd.is_file

    pretty_name = get_release_value('PRETTY_NAME')
    assert pretty_name

    version = get_release_value('VERSION')
    assert version

    variant = get_variant()

    if 'hardened' in variant:
        pytest.skip('Unable to validate motd in hardened images.')

    assert (
        motd.contains(pretty_name) or
        motd.contains(
            'SUSE Linux Enterprise Server {0}'.format(
                version.replace('-', ' ')
            )
        )
    )

    # check motd content for all segments
    def contains_registration_tools(motd_file):
        """Checks if any of the registration tools is mentioned in motd"""
        return (
            motd_file.contains('registercloudguest') or
            motd_file.contains('SUSEConnect') or
            motd_file.contains('transactional-update')
        )

    contain_results = []

    motd = host.file('/etc/motd')
    if motd.exists:
        contain_results.append(contains_registration_tools(motd))
    else:
        # check for every segment under the motd.d dir
        motd_dir = '/usr/lib/motd.d'
        motd_ls_cmd = f'ls {motd_dir}'
        result = host.run(motd_ls_cmd)

        segments = [filename for filename in result.stdout.strip().split('\n') if filename]  # NOQA
        for segment in sorted(segments):
            motd = host.file(f'{motd_dir}/{segment}')
            if motd.exists and motd.is_file:
                contain_results.append(contains_registration_tools(motd))

    if is_byos() and not is_suma_proxy():
        assert contain_results and any(contain_results)
    else:
        assert contain_results and not any(contain_results)



import pytest


def test_sles_haveged(check_service, get_release_value, is_sle_micro):
    version = get_release_value('VERSION')
    assert version

    have_haveged = ('12-SP5', '15-SP3')
    if version not in have_haveged:
        pytest.skip('haveged service is only in 12-SP5 and 15-SP3 images')

    if is_sle_micro() and float(version) >= 6.0:
        pytest.skip('haveged service is not in micro 6+ images')

    assert check_service('haveged')



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""img_proof GCE cloud unit tests."""

# Copyright (c) 2019 SUSE LLC. All rights reserved.
#
# This file is part of img_proof. img_proof provides an api and command line
# utilities for testing images in the Public Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest

from img_proof.ipa_gce import GCECloud
from img_proof.ipa_exceptions import GCECloudException

from unittest.mock import MagicMock, patch


class TestGCECloud(object):
    """Test GCE cloud class."""

    @patch('img_proof.ipa_gce.AuthorizedSession')
    @patch('img_proof.ipa_gce.service_account')
    @patch.object(GCECloud, '_validate_region')
    @patch('img_proof.ipa_gce.compute_v1')
    def setup_method(
        self,
        method,
        mock_compute_v1,
        mock_validate_region,
        mock_service_account,
        mock_auth_session
    ):
        """Set up kwargs dict."""
        self.kwargs = {
            'config': 'tests/data/config',
            'distro_name': 'SLES',
            'image_id': 'fakeimage',
            'no_default_test_dirs': True,
            'ssh_private_key_file': 'tests/data/ida_test',
            'test_files': ['test_image'],
            'custom_args': {
                'service_account_file': 'tests/gce/service-account.json'
            },
            'instance_options': 'STACK_TYPE=IPV4_ONLY'
        }

        self.instances_client = MagicMock()
        self.zone_ops_client = MagicMock()
        self.zones_client = MagicMock()
        self.networks_client = MagicMock()
        self.subnet_client = MagicMock()
        self.machine_type_client = MagicMock()
        self.images_client = MagicMock()
        self.disks_client = MagicMock()
        mock_compute_v1.InstancesClient.return_value = self.instances_client
        mock_compute_v1.ZoneOperationsClient.return_value = \
            self.zone_ops_client
        mock_compute_v1.ZonesClient.return_value = self.zones_client

        service_account = MagicMock()
        mock_service_account.Credentials.\
            from_service_account_file.return_value = service_account

        self.cloud = GCECloud(**self.kwargs)

    def test_gce_exception_required_args(self):
        """Test an exception is raised if required args missing."""
        self.kwargs['custom_args']['service_account_file'] = None
        self.kwargs['ssh_private_key_file'] = None

        # Test service account file required
        with pytest.raises(GCECloudException) as error:
            GCECloud(**self.kwargs)

        assert str(error.value) == \
            'Service account file is required to connect to GCE.'

        self.kwargs['custom_args']['service_account_file'] = \
            'tests/gce/service-account.json'

        # Test ssh private key file required
        with pytest.raises(GCECloudException) as error:
            GCECloud(**self.kwargs)

        assert str(error.value) == \
            'SSH private key file is required to connect to instance.'

        self.kwargs['ssh_private_key_file'] = 'tests/data/ida_test'

    @patch('img_proof.ipa_gce.service_account')
    def test_gce_get_service_account_info_invalid(self, mock_service_account):
        """Test get credentials method with invalid service account."""
        self.cloud.service_account_file = \
            'tests/gce/invalid-service-account.json'

        with pytest.raises(GCECloudException) as error:
            self.cloud._get_credentials()

        msg = 'Service account JSON file is invalid for GCE. ' \
            'client_email key is expected. See getting started ' \
            'docs for information on GCE configuration.'
        assert str(error.value) == msg

    @patch('img_proof.ipa_gce.AuthorizedSession')
    @patch('img_proof.ipa_gce.service_account')
    def test_gce_get_creds_invalid(
        self,
        mock_service_account,
        mock_auth_session
    ):
        """Test get credentials method with exception."""
        session = MagicMock()
        session.get.side_effect = Exception('Invalid creds!')
        mock_auth_session.return_value = session

        with pytest.raises(GCECloudException) as error:
            self.cloud._get_credentials()

        msg = 'GCP authentication failed: Invalid creds!'
        assert str(error.value) == msg

    def test_gce_get_instance(self):
        """Test gce get instance method."""
        instance = MagicMock()
        self.instances_client.get.return_value = instance

        val = self.cloud._get_instance()

        assert val == instance

        self.cloud.running_instance_id = 'test-instance'
        self.instances_client.get.side_effect = Exception(
            'test-instance cannot be found.'
        )

        with pytest.raises(GCECloudException) as error:
            self.cloud._get_instance()

        exc = "Unable to retrieve instance: test-instance cannot be found."
        assert str(error.value) == exc

    @patch('img_proof.ipa_gce.compute_v1')
    def test_gce_get_network(self, mock_compute_v1):
        """Test GCE get network method."""
        mock_compute_v1.NetworksClient.return_value = self.networks_client
        network = MagicMock()
        self.networks_client.get.return_value = network

        result = self.cloud._get_network('test-network')

        assert result == network

        self.networks_client.get.side_effect = Exception(
            'Resource test-network not found.'
        )

        msg = 'Unable to retrieve network: Resource test-network not found.'
        with pytest.raises(GCECloudException) as error:
            self.cloud._get_network('test-network')

        assert msg == str(error.value)

    @patch('img_proof.ipa_gce.compute_v1')
    def test_gce_get_subnet(self, mock_compute_v1):
        """Test GCE get subnetwork method."""
        mock_compute_v1.SubnetworksClient.return_value = self.subnet_client
        subnetwork = MagicMock()
        self.subnet_client.get.return_value = subnetwork

        self.cloud.region = 'us-west-1a'
        result = self.cloud._get_subnet('test-subnet')

        assert result == subnetwork

        self.subnet_client.get.side_effect = Exception(
            'Resource test-subnet not found.'
        )

        msg = 'Unable to retrieve subnet: Resource test-subnet not found.'
        with pytest.raises(GCECloudException) as error:
            self.cloud._get_subnet('test-subnet')

        assert msg == str(error.value)

    @patch('img_proof.ipa_gce.compute_v1')
    def test_gce_get_instance_type(self, mock_compute_v1):
        """Test GCE get instance type method."""
        mock_compute_v1.MachineTypesClient.return_value = \
            self.machine_type_client
        machine_type = MagicMock()
        self.machine_type_client.get.return_value = machine_type

        result = self.cloud._get_instance_type('n1-standard-1')
        assert result == machine_type

        self.machine_type_client.get.side_effect = Exception(
            'Resource n1-standard-1 not found.'
        )

        msg = 'Unable to retrieve instance type: ' \
              'Resource n1-standard-1 not found.'
        with pytest.raises(GCECloudException) as error:
            self.cloud._get_instance_type('n1-standard-1')

        assert msg == str(error.value)

    @patch('img_proof.ipa_gce.compute_v1')
    def test_gce_get_image(self, mock_compute_v1):
        """Test GCE get image method."""
        mock_compute_v1.ImagesClient.return_value = self.images_client
        image = MagicMock()
        self.images_client.get.return_value = image

        result = self.cloud._get_image('fake-image-20200202')
        assert result == image

        self.images_client.get.side_effect = Exception(
            'Resource fake-image-20200202 not found.'
        )

        msg = 'Unable to retrieve image: ' \
              'Resource fake-image-20200202 not found.'
        with pytest.raises(GCECloudException) as error:
            self.cloud._get_image('fake-image-20200202')

        assert msg == str(error.value)

    @patch('img_proof.ipa_gce.compute_v1')
    def test_gce_get_disk(self, mock_compute_v1):
        """Test GCE get image method."""
        mock_compute_v1.DisksClient.return_value = self.disks_client
        disk = MagicMock()
        self.disks_client.get.return_value = disk

        result = self.cloud._get_disk('disk12')
        assert result == disk

        self.disks_client.get.side_effect = Exception(
            'Resource disk12 not found.'
        )

        msg = 'Unable to retrieve disk: ' \
              'Resource disk12 not found.'
        with pytest.raises(GCECloudException) as error:
            self.cloud._get_disk('disk12')

        assert msg == str(error.value)

    @patch.object(GCECloud, '_get_subnet')
    def test_get_network_config(self, mock_get_subnet):
        subnet = MagicMock()
        subnet.self_link = 'projects/test/regions/us-west1/subnetworks/sub-123'
        subnet.network.self_link = 'projects/test/global/networks/network'

        mock_get_subnet.return_value = subnet

        subnet_config = self.cloud._get_network_config(
            'sub-123',
            use_gvnic=True
        )

        assert subnet_config['network'] == subnet.network.self_link
        assert subnet_config['subnetwork'] == subnet.self_link

    def test_get_shielded_instance_config(self):
        si_config = self.cloud.get_shielded_instance_config()

        assert si_config['enable_secure_boot'] is False
        assert si_config['enable_vtpm']
        assert si_config['enable_integrity_monitoring']

    def test_get_instance_config(self):
        config = self.cloud.get_instance_config(
            'instance123',
            'n1-standard-1',
            [{}],
            'service-account-123@email.com',
            'image123',
            'secretkey',
            50,
            'x86_64',
            shielded_instance_config={'enable_secure_boot': True},
            sev='SEV_SNP',
            use_gvnic=True
        )

        assert 'metadata' in config
        assert 'service_accounts' in config
        assert 'machine_type' in config
        assert 'disks' in config
        assert 'network_interfaces' in config
        assert 'name' in config
        assert 'shielded_instance_config' in config

    @patch.object(GCECloud, '_wait_on_instance')
    @patch.object(GCECloud, 'wait_for_extended_operation')
    @patch.object(GCECloud, '_get_network')
    @patch.object(GCECloud, '_get_image')
    @patch.object(GCECloud, '_get_instance_type')
    @patch('img_proof.ipa_utils.generate_instance_name')
    def test_gce_launch_instance(
        self,
        mock_generate_instance_name,
        mock_get_instance_type,
        mock_get_image,
        mock_get_network,
        mock_wait_on_operation,
        mock_wait_on_instance
    ):
        """Test GCE launch instance method."""
        mock_generate_instance_name.return_value = 'test-instance'
        network = MagicMock()
        network.self_link = 'projects/test/global/networks/net1'
        mock_get_network.return_value = network

        image = MagicMock()
        image.self_link = 'projects/test/global/images/img-123'
        mock_get_image.return_value = image

        inst_type = MagicMock()
        inst_type.self_link = 'zones/us-west1-a/machineTypes/n1-standard-1'
        mock_get_instance_type.return_value = inst_type

        mock_wait_on_operation.return_value = None

        operation = MagicMock()
        operation.name = 'operation123'
        self.instances_client.insert.return_value = operation

        self.cloud.region = 'us-west1-a'

        self.cloud._launch_instance()

        assert self.cloud.running_instance_id == 'test-instance'
        assert mock_wait_on_instance.call_count == 1

        # Exception on operation

        self.instances_client.insert.side_effect = Exception(
            'Create failed!'
        )

        with pytest.raises(GCECloudException) as error:
            self.cloud._launch_instance()

        assert 'Failed to launch instance: Create failed!' == str(error.value)

    @patch.object(GCECloud, '_get_disk')
    @patch.object(GCECloud, '_get_instance')
    def test_gce_set_image_id(self, mock_get_instance, mock_get_disk):
        """Test gce cloud set image id method."""
        instance = MagicMock()
        disk = MagicMock()
        disk.device_name = 'disk123'
        disk.boot = True
        disk.source = (
            'https://www.googleapis.com/compute/v1/projects/'
            'test/zones/us-west1-a/disks/disk123'
        )
        disk.source_image = 'projects/suse/global/images/opensuse-leap-15.0'
        instance.disks = [disk]
        mock_get_instance.return_value = instance
        mock_get_disk.return_value = disk

        self.cloud._set_image_id()

        assert self.cloud.image_id == 'opensuse-leap-15.0'
        assert mock_get_instance.call_count == 1
        assert mock_get_disk.call_count == 1

    def test_gce_validate_region(self):
        """Test gce cloud set image id method."""
        zones_obj = MagicMock()
        zones_obj.return_value = None
        self.zones_client.get.return_value = zones_obj

        with pytest.raises(GCECloudException) as error:
            self.cloud._validate_region()

        assert str(error.value) == \
            'Zone is required for GCE cloud framework: Example: us-west1-a'

        self.cloud.region = 'fake'

        with pytest.raises(GCECloudException) as error:
            self.cloud._validate_region()

        assert str(error.value) == \
            'fake is not a valid GCE zone. Example: us-west1-a'

    @patch.object(GCECloud, '_get_instance')
    def test_gce_is_instance_running(self, mock_get_instance):
        """Test gce cloud is instance runnning method."""
        instance = MagicMock()
        instance.status = 'RUNNING'
        mock_get_instance.return_value = instance
        assert self.cloud._is_instance_running()
        assert mock_get_instance.call_count == 1

        instance.status = 'TERMINATED'
        mock_get_instance.return_value = instance
        assert not self.cloud._is_instance_running()

    @patch.object(GCECloud, '_get_instance')
    def test_gce_set_instance_ip(self, mock_get_instance):
        """Test gce cloud set instance ip method."""
        interface = MagicMock()
        interface.network_i_p = None
        interface.access_configs = []
        instance = MagicMock()
        instance.network_interfaces = [interface]
        mock_get_instance.return_value = instance

        self.cloud.running_instance_id = 'test'

        with pytest.raises(GCECloudException) as error:
            self.cloud._set_instance_ip()

        assert str(error.value) == \
            'IP address for instance: test cannot be found.'
        assert mock_get_instance.call_count == 1

        interface.network_i_p = '10.0.0.0'
        mock_get_instance.return_value = instance
        self.cloud._set_instance_ip()

        assert self.cloud.instance_ip == '10.0.0.0'

    @patch.object(GCECloud, '_wait_on_instance')
    def test_gce_start_instance(self, mock_wait_on_instance):
        """Test gce start instance method."""
        mock_wait_on_instance.return_value = None
        self.instances_client.start.return_value = None

        self.cloud._start_instance()

        assert self.instances_client.start.call_count == 1

    @patch.object(GCECloud, '_wait_on_instance')
    def test_gce_stop_instance(self, mock_wait_on_instance):
        """Test gce stop instance method."""
        mock_wait_on_instance.return_value = None
        self.instances_client.stop.return_value = None

        self.cloud._stop_instance()

        assert self.instances_client.stop.call_count == 1

    def test_gce_terminate_instance(self):
        """Test gce terminate instance method."""
        self.instances_client.delete.return_value = None

        self.cloud._terminate_instance()
        assert self.instances_client.delete.call_count == 1

    def test_gce_get_console_log(self):
        """Test gce get console log method."""
        operation = MagicMock()
        operation.content = 'some output'
        self.instances_client.get_serial_port_output.return_value = operation

        self.cloud.get_console_log()
        assert self.instances_client.get_serial_port_output.call_count == 1

    @patch('img_proof.ipa_gce.time')
    def test_wait_on_operation(self, mock_time):
        self.cloud.service_account_project = 'test_project'
        self.cloud.region = 'us-west1-a'

        mock_time.sleep.return_value = None
        mock_time.time.return_value = 10

        operation = MagicMock()
        operation.status = 'DONE'
        self.zone_ops_client.get.return_value = operation

        result = self.cloud._wait_on_operation('operation213')
        assert result.status == 'DONE'
        assert self.zone_ops_client.get.call_count == 1

    def test_wait_for_extended_operation(self):
        warning = MagicMock()
        warning.code = '123'
        warning.message = 'Something is wrong!'

        operation = MagicMock()
        operation.error_code = '412'
        operation.error_message = 'Operation failed!'
        operation.warnings = [warning]
        operation.result.return_value = False

        with pytest.raises(GCECloudException):
            self.cloud.wait_for_extended_operation(operation)
